package org.diagbot.bigdata.common;

import org.apache.commons.lang3.StringUtils;
import org.diagbot.bigdata.util.BigDataConstants;
import org.diagbot.common.dao.model.StandardInfo;
import org.diagbot.common.service.StandardInfoService;
import org.diagbot.nlp.util.Constants;
import org.diagbot.nlp.util.NegativeEnum;

import javax.servlet.ServletContext;
import java.util.*;

public class ApplicationCacheUtil {

    public void putStandardInfoContext(ServletContext servletContext, StandardInfoService standardInfoService) {
        Map<String, Object> map = new HashMap<>(10, 0.8f);
        //词典疾病名称缓存
        map.put("categoryId", NegativeEnum.DISEASE.toString());
        this.putStandardInfoNameContext(servletContext, standardInfoService, map, BigDataConstants.standard_info_diag_cache);
        //症状名称缓存
        map.put("categoryId", NegativeEnum.SYMPTOM.toString());
        this.putStandardInfoNameContext(servletContext, standardInfoService, map, BigDataConstants.standard_info_symptom_cache);
        //同义词缓存
        this.putStandardInfoRelationNameContext(servletContext, standardInfoService, BigDataConstants.standard_info_relation_type_2, BigDataConstants.standard_info_synonym_cache);
        //分类关系
        this.putStandardInfoRelationNameContext(servletContext, standardInfoService, BigDataConstants.standard_info_relation_type_3, BigDataConstants.standard_info_type_cache);
        //分类关系树形结构存储    暂时只支持三级结构
        Map<String, String> standardInfoTypeMap = (Map<String, String>) servletContext.getAttribute(BigDataConstants.standard_info_type_cache);

        Map<String, Node> nodeMap = new HashMap<>();
        for (Map.Entry<String, String> entry : standardInfoTypeMap.entrySet()) {
            Node parent_node = nodeMap.get(entry.getValue());
            if (parent_node == null) {
                parent_node = new Node();
                parent_node.setName(entry.getValue());
            }
            Node child_node = nodeMap.get(entry.getKey());
            if (child_node == null) {
                child_node = new Node();
                child_node.setName(entry.getKey());
            }
            parent_node.add(parent_node, child_node);
            nodeMap.put(entry.getValue(), parent_node);
            nodeMap.put(entry.getKey(), child_node);
        }
        servletContext.setAttribute(BigDataConstants.standard_info_type_tree_cache, nodeMap);

    }

    public void putStandardInfoNameContext(ServletContext servletContext, StandardInfoService standardInfoService, Map<String, Object> map, String key) {
        Map<String, String> standardInfoMap = new HashMap<>(10, 0.8f);
        List<StandardInfo> list = standardInfoService.selectList(map);
        for (StandardInfo standardInfo : list) {
            standardInfoMap.put(standardInfo.getName(), standardInfo.getName());
        }
        servletContext.setAttribute(key, standardInfoMap);
    }

    public void putStandardInfoRelationNameContext(ServletContext servletContext, StandardInfoService standardInfoService, String relationType, String key) {
        Map<String, String> standardInfoSynonymMap = new HashMap<>(10, 0.8f);
        Map<String, Object> relationMap = new HashMap<>(10, 0.3f);
        if (StringUtils.isNotEmpty(relationType)) {
            relationMap.put("relationType", relationType);
        }
        List<StandardInfo> list = standardInfoService.selectNotNullRelationTerm(relationMap);
        for (StandardInfo standardInfo : list) {
            if (StringUtils.isNotEmpty(standardInfo.getRelationName())) {
                standardInfoSynonymMap.put(standardInfo.getName(), standardInfo.getRelationName());
            }
        }
        servletContext.setAttribute(key, standardInfoSynonymMap);
    }

    private void addChilds(Node root, Map<String, String> standardInfoTypeMap, List<String> hasAddList) {
        for (Node node : root.getChilds()) {
            for (Map.Entry<String, String> entry : standardInfoTypeMap.entrySet()) {
                if (node.getName().equals(entry.getValue())) {
                    node.add(node, entry.getKey());
                    hasAddList.add(entry.getKey());
                }
            }
        }
    }

    public class Node {
        private String name;
        private Node parent;
        private Set<Node> childs = new HashSet<>();

        public Node add(Node parent, String name) {
            Node node = new Node();
            node.setName(name);
            node.setParent(parent);
            parent.getChilds().add(node);
            return parent;
        }

        public Node add(Node parent, Node node) {
            node.setParent(parent);
            parent.getChilds().add(node);
            return parent;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public Set<Node> getChilds() {
            return childs;
        }

        public void setChilds(Set<Node> childs) {
            this.childs = childs;
        }

        public Node getParent() {
            return parent;
        }

        public void setParent(Node parent) {
            this.parent = parent;
        }
    }
}
