package org.diagbot.bigdata.work;

import org.algorithm.util.AlgorithmClassify;
import org.apache.commons.beanutils.BeanUtils;
import org.diagbot.bigdata.util.BigDataConstants;
import org.diagbot.nlp.feature.FeatureAnalyze;
import org.diagbot.nlp.feature.FeatureType;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName org.diagbot.bigdata.work.ParamsDataProxy
 * @Description TODO
 * @Author fyeman
 * @Date 2019/1/16/016 14:04
 * @Version 1.0
 **/
public class ParamsDataProxy {
    public void createSearchData(HttpServletRequest request, BigDataSearchData searchData) throws Exception {
        //消除空格
        if (searchData.getSymptom() != null) {
            searchData.setSymptom(searchData.getSymptom().trim());
        }
        if (searchData.getDiag() != null) {
            searchData.setDiag(searchData.getDiag().trim());
        }
        //计算年龄区间
        if (searchData.getAge() > 0) {
            searchData.setAge_start(searchData.getAge() - 5);
            searchData.setAge_end(searchData.getAge() + 5);
        }
        //默认查询门诊数据
        if (StringUtils.isEmpty(searchData.getResourceType())) {
            searchData.setResourceType(BigDataConstants.resource_type_o);
        }
        //一次推送多个类别信息
        String[] featureTypes = searchData.getFeatureType().split(",");
        searchData.setFeatureTypes(featureTypes);
        //featureType转算法模型类别
        searchData.setAlgorithmClassify(createAlgorithmClassify(searchData.getSysCode(), featureTypes, searchData.getDiag()));
        //获取入参中的特征信息
        FeatureAnalyze fa = new FeatureAnalyze();

        if (!StringUtils.isEmpty(searchData.getSymptom())) {
            List<Map<String, Object>> featuresList = fa.start(searchData.getSymptom(), FeatureType.FEATURE);
            paramFeatureInit(searchData, featuresList, "symptomFeatureList", FeatureType.FEATURE);
        }
    }

    /**
     * featureType转算法模型类型
     * @param sysCode
     * @param featureTypes
     * @param diag
     */
    private AlgorithmClassify[] createAlgorithmClassify(String sysCode, String[] featureTypes, String diag) {
        AlgorithmClassify[] classifies = new AlgorithmClassify[featureTypes.length];
        for (int i = 0; i < featureTypes.length; i++) {
            featureTypes[i] = convertFeatureType(sysCode, featureTypes[i]);
            if (featureTypes[i] != null) {
                //模型
                switch (FeatureType.parse(featureTypes[i])) {
                    case SYMPTOM:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_SYMPTOM;
                        } else {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG_SYMPTOM;
                        }
                        break;
                    case DIAG:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG;
                        }  else {
                            classifies[i] = null;
                        }
                        break;
                    case VITAL:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_VITAL;
                        } else {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG_VITAL;
                        }
                        break;
                    case LIS:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_LIS;
                        } else {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG_LIS;
                        }
                        break;
                    case PACS:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_PACS;
                        } else {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG_PACS;
                        }
                        break;
                    case TREAT:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_TREAT;
                        } else {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG_TREAT;
                        }
                        break;
                    case HISTORY:
                        if (StringUtils.isEmpty(diag)) {
                            classifies[i] = AlgorithmClassify.NEURAL_HISTORY;
                        } else {
                            classifies[i] = AlgorithmClassify.NEURAL_DIAG_HISTORY;
                        }
                        break;
                }
            }
        }
        return classifies;
    }

    /**
     * 外部系统featureType需要转化为大数据定义的featureType
     * @param sysCode
     * @param featureType
     * @return
     */
    private String convertFeatureType(String sysCode, String featureType) {
        if (StringUtils.isEmpty(sysCode) || sysCode.equals("1")) {
            if ("1".equals(featureType)) {
                return BigDataConstants.feature_type_symptom;
            }
            if ("7".equals(featureType)) {
                return BigDataConstants.feature_type_diag;
            }
            if ("4".equals(featureType)) {
                return BigDataConstants.feature_type_vital;
            }
            if ("5".equals(featureType)) {
                return BigDataConstants.feature_type_lis;
            }
            if ("6".equals(featureType)) {
                return BigDataConstants.feature_type_pacs;
            }
            if ("3".equals(featureType)) {
                return BigDataConstants.feature_type_history;
            }
            if ("8".equals(featureType)) {
                return BigDataConstants.feature_type_treat;
            }
            return null;
        }
        return featureType;
    }

    /**
     * 推送模型入参
     *
     * @param searchData
     * @param property_list
     * @param featureType
     * @throws Exception
     */
    private void paramFeatureInit(BigDataSearchData searchData, List<Map<String, Object>> featuresList, String property_list, FeatureType featureType) throws Exception {
        if (featuresList != null && featuresList.size() > 0) {
            BeanUtils.setProperty(searchData, property_list, featuresList);
            Map<String, String> map = new HashMap<>();
            Map<String, Object> featureMap = null;
            for (int i = 0; i < featuresList.size(); i++) {
                featureMap = featuresList.get(i);
                for (Map.Entry<String, Object> entry : featureMap.entrySet()) {
                    map.put(entry.getKey(), String.valueOf(entry.getValue()));
                }
                map.put("featureType", String.valueOf(featureMap.get("feature_type")));
                map.put("featureName", String.valueOf(featureMap.get("feature_name")));
                searchData.getInputs().put(map.get("feature_name"), map);
            }
        }
    }
}
